import re

import requests

from orthanc_ext.orthanc import OrthancApiHandler

orthanc = OrthancApiHandler()


def test_change_type_list_should_be_complete():
    events = get_type_enum_values("OrthancPluginChangeType")

    assert len(events) > 16
    for event in events:
        assert orthanc.ChangeType.__dict__.get(
            event) is not None, f"'{event} should be added on {orthanc.ChangeType}"


def test_resource_type_list_should_be_complete():
    events = get_type_enum_values("OrthancPluginResourceType")

    assert len(events) > 4
    for event in events:
        assert orthanc.ResourceType.__dict__.get(
            event) is not None, f"'{event} should be added on {orthanc.ResourceType}"


def get_type_enum_values(type_under_test):
    resp = requests.get(
        f'https://hg.orthanc-server.com/orthanc-python/raw-file/tip/Sources/Autogenerated/sdk_{type_under_test}.impl.h')
    resp.raise_for_status()

    return re.findall(f'sdk_{type_under_test}_Type.tp_dict, "([A-Z_]+)"', resp.text)




import threading
from collections.abc import Iterable
from dataclasses import dataclass


def register_event_handlers(event_handlers, orthanc_module):
    @dataclass
    class ChangeEvent:
        change_type: int
        resource_type: int
        resource_id: str

        def __str__(self):
            return f"ChangeEvent(change_type={event_types.get(self.change_type)}, resource_type={self.resource_type}, resource_id='{self.resource_id}')"

    def ensure_iterable(v):
        return v if isinstance(v, Iterable) else [v]

    def can_hash(k):
        try:
            return hash(k)
        except TypeError:
            return False

    event_types = {v: k for k, v in orthanc_module.ChangeType.__dict__.items() if can_hash(v)}
    event_handlers = {k: ensure_iterable(v) for k, v in event_handlers.items()}

    def unhandled_event_logger(event, orthanc):
        orthanc.LogInfo(f'no handler registered for {event_types[event.change_type]}')

    def OnChange(change_type, resource_type, resource_id):
        handlers = event_handlers.get(change_type, [unhandled_event_logger])
        threads = []
        for handler in handlers:
            event = ChangeEvent(change_type, resource_type, resource_id)
            threads.append(threading.Timer(0, function=handler, args=(event, orthanc_module)))

        for thread in threads:
            thread.start()
            thread.join()

    orthanc_module.RegisterOnChangeCallback(OnChange)



"""
This module implements the Orthanc Python plugin API to run requests against an external Orthanc instance.

This will allow you quickly evolve your python scripts and make them easy to integration test as well.
"""
from dataclasses import dataclass

import requests

import logging


class OrthancApiHandler(object):
    class ResourceType:
        PATIENT = 0
        STUDY = 1
        SERIES = 2
        INSTANCE = 3
        NONE = 4

    # redefine to make this type available for unit tests outside the Orthanc Python plugin.
    # https://hg.orthanc-server.com/orthanc-python/file/tip/Sources/Autogenerated/sdk_OrthancPluginChangeType.impl.h
    class ChangeType:
        COMPLETED_SERIES = 0
        DELETED = 1
        NEW_CHILD_INSTANCE = 2
        NEW_INSTANCE = 3
        NEW_PATIENT = 4
        NEW_SERIES = 5
        NEW_STUDY = 6
        STABLE_PATIENT = 7
        STABLE_SERIES = 8
        STABLE_STUDY = 9
        ORTHANC_STARTED = 10
        ORTHANC_STOPPED = 11
        UPDATED_ATTACHMENT = 12
        UPDATED_METADATA = 13
        UPDATED_PEERS = 14
        UPDATED_MODALITIES = 15
        JOB_SUBMITTED = 16
        JOB_SUCCESS = 17
        JOB_FAILURE = 18

        # not defined by orthanc
        UNKNOWN = 999

    def __init__(self, session=requests.Session()):
        self.session = session
        self.base_url = "https://localhost:8042"

        def default_callback():
            raise NotImplemented("change_callback")

        self.change_callback = default_callback

    def LogWarning(self, message):
        logging.log(logging.WARNING, message)

    def LogInfo(self, message):
        logging.log(logging.INFO, message)

    def HttpPost(self, path, body, headers):
        response = self.session.post(path, body, headers=headers, verify=False)
        response.raise_for_status()
        return response.text

    def RestApiPost(self, path, body):
        calculated_path = f'{self.base_url}{path}'

        response = self.session.post(calculated_path, body, verify=False)
        response.raise_for_status()
        return response.text

    def RestApiGet(self, path):
        response = self.session.get(f'{self.base_url}{path}', verify=False)
        response.raise_for_status()
        return response.text

    def RestApiDelete(self, path):
        response = self.session.delete(f'{self.base_url}{path}', verify=False)
        response.raise_for_status()
        return response.text

    def RegisterOnChangeCallback(self, change_callback):
        self.change_callback = change_callback

    def on_change(self, change_type, resource_type, resource_id):
        return self.change_callback(change_type, resource_type, resource_id)



import responses
from orthanc_ext.orthanc import OrthancApiHandler

orthanc = OrthancApiHandler()


@responses.activate
def test_delete_should_forward_to_requests():
    responses.add(responses.DELETE, 'https://localhost:8042/test',
                  json={}, status=202)

    orthanc.RestApiDelete('/test')



import logging
import re
from time import sleep

import requests
from attr import dataclass

from orthanc_ext import event_dispatcher
from orthanc_ext.orthanc import OrthancApiHandler

orthanc = OrthancApiHandler()


@dataclass
class ChangeEvent:
    change: orthanc.ChangeType = orthanc.ChangeType.UNKNOWN
    resource_type: int = orthanc.ResourceType.NONE
    resource_id: str = None
    orthanc = None


def capture(event):
    def capture_impl(incoming_event, local_orthanc):
        event.change = incoming_event.change_type
        event.resource_type = incoming_event.resource_type
        event.resource_id = incoming_event.resource_id
        event.orthanc = local_orthanc

    return capture_impl


def test_registered_callback_should_be_triggered_on_change_event():
    event = ChangeEvent()

    event_dispatcher.register_event_handlers({
        orthanc.ChangeType.STABLE_STUDY: capture(event)
    }, orthanc_module=orthanc)

    orthanc.on_change(orthanc.ChangeType.STABLE_STUDY, orthanc.ResourceType.STUDY, "resource-uuid")

    assert event.resource_id == "resource-uuid"
    assert event.resource_type == orthanc.ResourceType.STUDY
    assert event.orthanc is not None


def test_all_registered_callbacks_should_be_triggered_on_change_event():
    event1 = ChangeEvent()
    event2 = ChangeEvent()

    event_dispatcher.register_event_handlers({
        orthanc.ChangeType.STABLE_STUDY: [capture(event1), capture(event2)]
    }, orthanc_module=orthanc)

    orthanc.on_change(orthanc.ChangeType.STABLE_STUDY, orthanc.ResourceType.STUDY, "resource-uuid")

    assert event1.resource_id is not None
    assert event2.resource_id is not None


def test_no_registered_callbacks_should_be_reported_in_on_change_event(caplog):
    caplog.set_level(logging.INFO)

    event_dispatcher.register_event_handlers({}, orthanc_module=orthanc)
    orthanc.on_change(orthanc.ChangeType.ORTHANC_STARTED, '', '')

    assert "no handler registered for ORTHANC_STARTED" in caplog.text



"""Test entry point script for Orthanc Python Plugin."""
from orthanc_ext import event_dispatcher

# provided by the plugin runtime.
import orthanc


def log_event():
    def log_event_impl(event, orthanc):
        orthanc.LogWarning(f"orthanc '{event}' event handled!")

    return log_event_impl


def start_maintenance_cycle(event, orthanc):
    orthanc.LogWarning(f"do something special on {event}")


event_dispatcher.register_event_handlers({
    orthanc.ChangeType.ORTHANC_STARTED: [log_event(), start_maintenance_cycle()],
    orthanc.ChangeType.ORTHANC_STOPPED: log_event()
}, orthanc_module=orthanc)
